function [Ybus,Node,indx_n,Phase_Nodes,N_ph_VR] = Admittance_Build(BaseMVA,BasekV,Source,branches,conf,caps,regs)
% Check the number of input arguments
% Source: the name of the node representing the source (substation)
% branches: a cell array containing From-To node names for each branch,
% branch length (ft), and branch configuration.
% conf: contains all configurations in the current system.
% caps: shunt capacitors (per phase kVAR)
% regs: voltage regulators. The first two columns are the two end nodes for
% the regulator and the last three columns are for the initial tap
% position. No tap adjustment procedure is considered in this version.
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

if nargin ~= 7
    error('Not Enough/Too Many Inputs!')
end

Nb = length(branches(:,1));% number of branches

Fr = branches(:,1);
To = branches(:,2);
% Find all the existing nodes in the system
dummy1 = [Fr;To];
[~,idx]=unique(dummy1,'stable');
dummy1 =  dummy1(idx);
% Find the source node
a_ind = find(strcmp(dummy1,Source),1);
dummy1(a_ind) = [];dummy1 = [Source;dummy1];% brings source node to the top

Nn = length(dummy1);% Number of total nodes including the source
if ~isempty(regs)
    N_VR = length(regs(:,1));% number of voltage regulators
    a_VR = zeros(N_VR,1);
    for k = 1:N_VR
        a_VR(k) = find(strcmp(branches(:,4),regs{k,3}),1);
    end
else
    N_VR = 0;
    a_VR = [];
end
N_ph_VR = 0;% Calculating the total number of phases for VRs
if N_VR>0
    for k = 1:N_VR
        N_ph_VR = N_ph_VR + length(regs{k,4});
    end
end
Y = cell(Nn+N_VR,Nn+N_VR);% initialize augmented Ybus, assuming every line section is three-phase
Bsh = repmat({zeros(3)},Nn,1);
% Adding shunt capacitors to the shunt elements
if ~isempty(caps)
    N_C = length(caps(:,1));%Number of shunt capacitors
else
    N_C = 0;
end

if N_C>0
    for k = 1:N_C
        Bsh{strcmp(dummy1(:,1),caps{k,1}),1} = 1i*(BaseMVA/1000)*diag([caps{k,2},caps{k,3},caps{k,4}]);
    end
end

S_loop = 1:Nb;S_loop(a_VR) = [];
for k = S_loop
    k1 = branches{k,4};
    %         Feet to Mile conversion, determining the branch impedance and
    %         equivalent shunt admittance in per-unit
    Z = 0.000189394*cell2mat(branches(k,3))*(BaseMVA/BasekV^2)*(conf.(k1).('R') + 1i*conf.(k1).('X'));
    B = 1i*1e-6*0.000189394*cell2mat(branches(k,3))*(BasekV^2/BaseMVA).*conf.(k1).('B');
    % 1 foot = 0.000189394 miles;
    idx1 = find(strcmp(dummy1, Fr(k)));% Find the corresponding node in dummy1
    idx2 = find(strcmp(dummy1, To(k)));
    switch conf.(k1).('config')
        case {'ABC','ACB','BCA','BAC','CAB','CBA'}
            Y{idx1,idx2} = inv(Z);
            Bsh{idx1} = Bsh{idx1} + B./2;Bsh{idx2} = Bsh{idx2} + B./2;
        case {'AB','BA'}
            Z(3,:) = [];Z(:,3) = [];B(3,:) = [];B(:,3) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [y_dum,[0;0];[0,0,0]];
            Bsh{idx1} = Bsh{idx1} + [B/2,[0;0];[0,0,0]];Bsh{idx2} = Bsh{idx2} + [B/2,[0;0];[0,0,0]];
        case {'AC','CA'}
            Z(2,:) = [];Z(:,2) = [];B(2,:) = [];B(:,2) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [y_dum(1,1),0,y_dum(1,2);0,0,0;y_dum(2,1),0,y_dum(2,2)];
            Bsh{idx1} = Bsh{idx1} + [B(1,1)/2,0,B(1,2)/2;0,0,0;B(2,1)/2,0,B(2,2)/2];
            Bsh{idx2} = Bsh{idx2} + [B(1,1)/2,0,B(1,2)/2;0,0,0;B(2,1)/2,0,B(2,2)/2];
        case {'BC','CB'}
            Z(1,:) = [];Z(:,1) = [];B(1,:) = [];B(:,1) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [[0,0,0];[0;0],y_dum];
            Bsh{idx1} = Bsh{idx1} + [[0,0,0];[0;0],B/2];
            Bsh{idx2} = Bsh{idx2} + [[0,0,0];[0;0],B/2];
        case 'A'
            Z = Z(1,1);B = B(1,1);
            Y{idx1,idx2} = [1/Z,0,0;0,0,0;0,0,0];
            Bsh{idx1} = Bsh{idx1} + [B/2,0,0;0,0,0;0,0,0];
            Bsh{idx2} = Bsh{idx2} + [B/2,0,0;0,0,0;0,0,0];
        case 'B'
            Z = Z(2,2);B = B(2,2);
            Y{idx1,idx2} = [0,0,0;0,1/Z,0;0,0,0];
            Bsh{idx1} = Bsh{idx1} + [0,0,0;0,B/2,0;0,0,0];
            Bsh{idx2} = Bsh{idx2} + [0,0,0;0,B/2,0;0,0,0];
        case 'C'
            Z = Z(3,3);B = B(3,3);
            Y{idx1,idx2} = [0,0,0;0,0,0;0,0,1/Z];
            Bsh{idx1} = Bsh{idx1} + [0,0,0;0,0,0;0,0,B/2];
            Bsh{idx2} = Bsh{idx2} + [0,0,0;0,0,0;0,0,B/2];
    end
    Y{idx2,idx1} = Y{idx1,idx2};
end

% Calculate diagonal elements
for k = 1:Nn
    aa = cell2mat(Y(k,:));
    bb = zeros(3);
    for k1 = 1:length(aa(1,:))/3
        bb = bb + aa(:,3*k1-2:3*k1);
    end
    Y{k,k} = -bb-Bsh{k,1};
end
% Adding the augmented values for the voltage regulators to the admittance
% matrix
if N_VR>0
    for k = 1:N_VR
        VR_F = strcmp(dummy1(:,1),regs{k,1});
        VR_T = strcmp(dummy1(:,1),regs{k,2});
        Y{Nn+k,VR_F} = -diag(cell2mat(regs(k,15:17)));
        Y{Nn+k,VR_T} = eye(3);
        Y{VR_F,Nn+k} = Y{Nn+k,VR_F};
        Y{VR_T,Nn+k} = Y{Nn+k,VR_T};
    end
end
Nv = cell(Nn,1);
% determine the phase variables at each node
for k = 1:Nn
    con1 = branches(strcmp(branches(:,1),dummy1(k)),4);
    con2 = branches(strcmp(branches(:,2),dummy1(k)),4);
    con12 = [con1;con2];% list of all the phase configs connected to this node
    if length(con12) == 1
        Nv{k,1} = conf.(char(con12)).config;
    elseif ~isempty(con12)
        con0 = cell(length(con12),2);
        for k1 = 1:length(con12)
            con0{k1,1} = conf.(char(con12(k1))).config;
            con0{k1,2} = length(con0{k1,1});
        end

        [~,b1] = max(cell2mat(con0(:,2)));
        Nv{k,1} = con0{b1,1};       
    end
end

Phase_Nodes = Nv;
N_Vol = length(strjoin(Nv','')) - 3;% number of total unknown phase voltages (source is taken out)
% Find index of non-existing phase voltages to be removed from the Ybus
indx_n = zeros(3*Nn-N_Vol-3,1);
m1 = 1;
for k = 1:Nn
    if length(Nv{k,1}) == 2
        switch Nv{k,1}
            case {'AB','BA'}
            indx_n(m1) = 3*k;
            m1 = m1+1;
            case {'AC','CA'}
            indx_n(m1) = 3*k-1;
            m1 = m1+1;
            case {'BC','CB'}
            indx_n(m1) = 3*k-2;
            m1 = m1+1;
        end
    elseif length(Nv{k,1}) == 1
        switch Nv{k,1}
            case 'A'
            indx_n(m1) = 3*k-1;
            indx_n(m1+1) = 3*k;
            m1 = m1+2;
            case 'B'
            indx_n(m1) = 3*k-2;
            indx_n(m1+1) = 3*k;
            m1 = m1+2;
            case 'C'
            indx_n(m1) = 3*k-2;
            indx_n(m1+1) = 3*k-1;
            m1 = m1+2;
        end
    end
end
        
        
for k1 = 1:Nn+N_VR
    for k2 = 1:k1
        if isempty(Y{k1,k2})
        Y{k1,k2} = zeros(3);
        Y{k2,k1} = zeros(3);
        end
    end
end

% Remove non-existing phases pertained to the VRs from augmented Ybus
if N_VR>0
    indx_VR = zeros(3*N_VR-N_ph_VR,1);
    m1 = 1;
    for k = 1:N_VR
        switch regs{k,4}
            case 'AB'
                indx_VR(m1) = 3*k;
                m1 = m1+1;
            case 'AC'
                indx_VR(m1) = 3*k-1;
                m1 = m1+1;
            case 'BC'
                indx_VR(m1) = 3*k-2;
                m1 = m1+1;
            case 'A'
                indx_VR(m1) = 3*k-1;
                indx_VR(m1+1) = 3*k;
                m1 = m1+2;
            case 'B'
                indx_VR(m1) = 3*k-2;
                indx_VR(m1+1) = 3*k;
                m1 = m1+2;
            case 'C'
                indx_VR(m1) = 3*k-2;
                indx_VR(m1+1) = 3*k-1;
                m1 = m1+2;
        end
    end
    indx_VR = indx_VR + 3*Nn;
else
    indx_VR = [];
end


Y = cell2mat(Y);
Y([indx_n;indx_VR],:) = [];Y(:,[indx_n;indx_VR]) = [];

Ybus = Y;
Node = dummy1;

end