function [results] = CIM_3Ph(BaseMVA,BasekV,Source,branches,loads,caps,regs,conf,V0)
% This function runs a power flow routine using the current injection
% method based on a fixed-point iteration. Details are given in the 
% following publication:
% H. Ahmadi, J.R. Marti, A. von Meier, "A linear power flow formulation for
% three-phase distribution systems," submitted to the IEEE Transactions on
% Power Systems, 2015.
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

% If no capacitor or voltage regulator exist, an empty input is given. 
% Call the routine to form the admittance matrix
[Ybus,Nodes,indx_n,Phase_Nodes,N_ph_VR] = Admittance_Build(BaseMVA,BasekV,Source,branches,conf,caps,regs);
Nn = length(Nodes)-1;% Number of nodes by removing the source node
% Take out the rows and columns associated with the voltage regulators in
% the augmented admittance matrix
G = -real(Ybus(1:end-N_ph_VR,1:end-N_ph_VR)); 
B = -imag(Ybus(1:end-N_ph_VR,1:end-N_ph_VR));

indx_n = indx_n-3;% Take out the substation nodes
% Initialize the nodal voltages in rectangular coordinates
Vre0 = V0*[cos(0);cos(-2*pi/3);cos(2*pi/3)];
Vim0 = V0*[sin(0);sin(-2*pi/3);sin(2*pi/3)];
Vre = repmat(Vre0,Nn,1);Vim = repmat(Vim0,Nn,1);
% Remove variables that do not exist
Vre(indx_n) = [];
Vim(indx_n) = [];
Nvar = length(Vre);% Number of total variables
% Initialize the auxiliary variables for voltage regulators
Ireg_re = zeros(N_ph_VR,1);
Ireg_im = zeros(N_ph_VR,1);

Niter = 6;% Number of fixed-point iterations
Vabc_m = zeros(Nvar,Niter);Vabc_a = Vabc_m;
% Take out the first triplet rows and columns associated with the source
yy = [G(4:end,4:end) -B(4:end,4:end);B(4:end,4:end) G(4:end,4:end)];
% Form the augmented matrix
A_aug = real(Ybus(4:end-N_ph_VR,end-N_ph_VR+1:end));
Ainv = [yy blkdiag(A_aug,A_aug);blkdiag(A_aug',A_aug'),zeros(2*N_ph_VR)];
Ainv = sparse(Ainv);% make the matrix into sparse form

for k = 1:Niter
    [Ire,Iim] = Load_Current(BaseMVA,loads,Nodes,indx_n,Phase_Nodes,Vre(:,k),Vim(:,k));
    Iinj1 = -G(4:end,1:3)*Vre0+B(4:end,1:3)*Vim0-Ire;
    Iinj2 = -G(4:end,1:3)*Vim0-B(4:end,1:3)*Vre0-Iim;
    X = Ainv\[Iinj1;Iinj2;zeros(2*N_ph_VR,1)];
    Vre(:,k+1) = X(1:Nvar);
    Vim(:,k+1) = X(Nvar+1:2*Nvar);
    if N_ph_VR>0
        Ireg_re(:,k+1) = X(2*Nvar+1:2*Nvar+N_ph_VR);
        Ireg_im(:,k+1) = X(2*Nvar+N_ph_VR+1:2*Nvar+2*N_ph_VR);
    end
    Vabc_m(:,k) = sqrt(Vre(:,k+1).^2 + Vim(:,k+1).^2);
    Vabc_a(:,k) = atan2d(Vim(:,k+1),Vre(:,k+1));
end


results = cell(Nn+2,7);
results(1,:) = {'#Node','Vm_A','Vm_B','Vm_C','Va_A','Va_B','Va_C'};
results(2:end,1) = Nodes;
results(2,2:end) = {1,1,1,0,-120,120};
% If voltage magnitudes are needed in actual values not in p.u.
% Vabc_m = BasekV*sqrt(3)*Vabc_m;

Vm = cell(3*Nn,1);
Va = cell(3*Nn,1);
m = 1;
for k = 1:3*Nn
    if any(indx_n==k)
        Vm{k} = [];
        Va{k} = [];
    else
        Vm{k} = Vabc_m(m,end);
        Va{k} = Vabc_a(m,end);
        m = m+1;
    end
    if ~mod(k,3)
        results(2+k/3,2:4) = Vm(k-2:k);
        results(2+k/3,5:7) = Va(k-2:k);
    end
end

% if the results should be displayed in the command window
% display(results)
