function [results] = LPF_3Ph(BaseMVA,BasekV,Source,branches,loads,caps,regs,conf,V0)
% This function returns the power flow results obtained using the
% three-phase linear power flow (3LPF) formulation. Details are given in
% the following publication:
% H. Ahmadi, J.R. Marti, A. von Meier, "A linear power flow formulation for
% three-phase distribution systems," submitted to the IEEE Transactions on
% Power Systems, 2015.
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

% Following the standard IEEE test case formats, loads are modeled based on
% their configuration (Y/D) and voltage dependence (PQ/Z/I). This loop
% redefines this data into a more general format. Columns 9 to 14 for each
% row in "loads" stand for the coefficients in the ZIP model of loads. The
% first three are for active power and the second three are for reactive 
% power in the following order: (C_Z, C_I, C_P). 
for k = 1:length(loads(:,1))
    switch loads{k,2}
        case 'Y-PQ'
            loads{k,2} = 'Y';
            loads(k,9:14) = {0 0 1 0 0 1};
        case 'Y-I'
            loads{k,2} = 'Y';
            loads(k,9:14) = {0 1 0 0 1 0};
        case 'Y-Z'
            loads{k,2} = 'Y';
            loads(k,9:14) = {1 0 0 1 0 0};
        case 'D-PQ'
            loads{k,2} = 'D';
            loads(k,9:14) = {0 0 1 0 0 1};
        case 'D-I'
            loads{k,2} = 'D';
            loads(k,9:14) = {0 1 0 0 1 0};
        case 'D-Z'
            loads{k,2} = 'D';
            loads(k,9:14) = {1 0 0 1 0 0};    
    end
end
% Call the function to calculate the admittance matrix
[GG1,GG2,BB1,BB2,Ire_0,Iim_0,Nodes,indx_n,~,N_ph_VR] = Admittance_Build_LPF(BaseMVA,BasekV,Source,branches,conf,loads,caps,regs);
Nn = length(Nodes)-1;% Number of nodes by removing the source node
indx_n = indx_n-3;% take out the elements associated with the substation

Vre0 = V0*[cos(0);cos(-2*pi/3);cos(2*pi/3)];
Vim0 = V0*[sin(0);sin(-2*pi/3);sin(2*pi/3)];
Nvar = 3*Nn-length(indx_n);

A = [GG1(4:end-N_ph_VR,4:end-N_ph_VR) BB1(4:end-N_ph_VR,4:end-N_ph_VR);BB2(4:end-N_ph_VR,4:end-N_ph_VR) GG2(4:end-N_ph_VR,4:end-N_ph_VR)];
a1 = GG1(4:end-N_ph_VR,end-N_ph_VR+1:end);z1 = zeros(size(a1));
az = -[a1,z1;z1,a1];
A = [A,az;az',zeros(size(az,2))];
Ire = -GG1(4:end-N_ph_VR,1:3)*Vre0-BB1(4:end-N_ph_VR,1:3)*Vim0-Ire_0;
Iim = -GG1(4:end-N_ph_VR,1:3)*Vim0+BB1(4:end-N_ph_VR,1:3)*Vre0-Iim_0;
V = A\[Ire;Iim;zeros(2*N_ph_VR,1)];       
Vre = V(1:Nvar);Vim = V(Nvar+1:2*Nvar);
Vabc_m = sqrt(Vre.^2 + Vim.^2);
Vabc_a = atan2d(Vim,Vre);
% Activate this line for voltage magnitudes in kV
% Vabc_m = BasekV*sqrt(3)*Vabc_m;
results = cell(Nn+2,7);
results(1,:) = {'#Node','Vm_A','Vm_B','Vm_C','Va_A','Va_B','Va_C'};
results(2:end,1) = Nodes;
results(2,2:end) = {1,1,1,0,-120,120};

Vm = cell(3*Nn,1);
Va = cell(3*Nn,1);
m = 1;
for k = 1:3*Nn
    if any(indx_n==k)
        Vm{k} = [];
        Va{k} = [];
    else
        Vm{k} = Vabc_m(m,end);
        Va{k} = Vabc_a(m,end);
        m = m+1;
    end
    if ~mod(k,3)
        results(2+k/3,2:4) = Vm(k-2:k);
        results(2+k/3,5:7) = Va(k-2:k);
    end
end

% Activate this line to display the results in the command window
% display(results)

end
